/* 
Copyright (C) 2014 - 2017 Texas Instruments Incorporated - http://www.ti.com/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the
distribution.

Neither the name of Texas Instruments Incorporated nor the names of
its contributors may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ======== peripherals.c ========
 *
 */
#include <MSP430.h>
#include <stdint.h>
#include "descriptors.h"

#ifndef _PERIPHERALS_H_
#define _PERIPHERALS_H_

#ifdef __cplusplus
extern "C"
{
#endif

// Maximal host system delay
#define MAX_SYS_DELAY   20  // ms

// Size of Rx/Tx UART Buffers
// Value should be big enough to cover max system delay
#define RX_BUFFER_SIZE   1280             // max_baud * SYS_DELAY ms/1000/8bit
                                          // This buffer size is allocated to store
                                          // data received from BSL target 
                                          // "RX" term is used because this 
                                          // would be the received data from
                                          // the BSL target
#define TX_BUFFER_SIZE   MAX_PACKET_SIZE  // Don't modify this value!!!
                                          // This buffer size is allocated to store
                                          // data received from USBCDC (host)
                                          // "TX" term is used because this 
                                          // would be the transmitted data from
                                          // Rocket to the BSL target
#define BSL_BUFFER_SIZE  266              // Maximum size of BSL command buffer

enum active_peripherals
{ None, 
  UART, 
  I2C,
  SPI
};

extern enum active_peripherals active_peripheral;
extern uint8_t rxBuffer[RX_BUFFER_SIZE]; // buffer of received data from BSL target
extern uint8_t txBuffer[TX_BUFFER_SIZE]; // buffer of received data from USBCDC
extern uint8_t *txBuffer_read_ptr;  // pointer used in rxBuffer
extern uint8_t *txBuffer_write_ptr; // pointer used in txBuffer


/*** Baudrate configuration ***************************************************/

uint8_t BaudrateSelect(uint32_t lBaudrate);
extern uint32_t current_baudrate;
void disable_peripherals();

/*** LEDs macros and function *************************************************/

#define LED_POUT         P1OUT
#define LED1_PIN         BIT0
#define LED2_PIN         BIT1
#define LED1_ON          {LED_POUT |= LED1_PIN;}
#define LED1_OFF         {LED_POUT &= ~LED1_PIN;}
#define LED1_TOGGLE      {LED_POUT ^= LED1_PIN;}
#define LED2_ON          {LED_POUT |= LED2_PIN;}
#define LED2_OFF         {LED_POUT &= ~LED2_PIN;}
#define LED2_TOGGLE      {LED_POUT ^= LED2_PIN;}

#define LED_BLINK_DELAY  5000000
#define LED_BLINK_COUNT  3

void startup_led_sequence();  // Blink the LEDs

/*** BSL entry and exit sequence **********************************************/

#define ENTRY_SEQ_PDIR         PJDIR
#define ENTRY_SEQ_POUT         PJOUT
#define RESET_PIN              BIT1
#define TEST_PIN               BIT2
#define TCK_PIN                BIT3

#define RESET_HIGH             {ENTRY_SEQ_POUT |= RESET_PIN;}
#define RESET_LOW              {ENTRY_SEQ_POUT &= ~RESET_PIN;}
#define TEST_HIGH              {ENTRY_SEQ_POUT |= TEST_PIN;}
#define TEST_LOW               {ENTRY_SEQ_POUT &= ~TEST_PIN;}
#define TCK_HIGH               {ENTRY_SEQ_POUT |= TCK_PIN;}
#define TCK_LOW                {ENTRY_SEQ_POUT &= ~TCK_PIN;}

#define DELAY_12MS             12    // Timing for Pin Toggling Legacy BSL Entry Sequence
#define INVOKE_DELAY_100US     2000  // Timing for Pin Toggling BSL Entry Sequence
#define INVOKE_DELAY_1MS       20000 // Timing for Pin Toggling Legacy BSL Entry Sequence
#define INVOKE_DELAY_2US       40
#define INVOKE_DELAY_110US     2200
#define INVOKE_DELAY_10US      200
#define BSL_EXIT_SEQUENCE_TIME 1000 // Timing for Pin Toggling BSL Exit Sequence

void BSL_invoke_sequence();         // Toggle RST/TST/TCK pins for BSL invoke
void BSL_delay_ms(uint8_t time);    // Microsecond delay function for Legacy BSL
void BSL_legacy_invoke_sequence();  // Toggle RST/TST/TCK pins for Legacy BSL invoke
void BSL_reset_sequence();          // Reset sequence to leave BSL (JTAG20 workaround)

/*** BSL Commands Macros ******************************************************/

#define BSL_STD_CMD_HEADER     0x80  // header byte of MSP430 and MSP432P BSL 
#define BSL_CUSTOM_CMD_HEADER  0xA0  // header byte of custom command

#define BSL_RESPONSE_ACK       0x00  // ACK value from all MSP430 and MSP432
#define BSL_RESPONSE_ACK_E4xx  0xCC  // For MSP432E, after BSL_RESPONSE_ACK
                                     // comes the next byte of 0xCC

#define BSL_CMD_RX_BLOCK_FAST  0x1B  // BSL command header byte that does not 
                                     // require ACK from the BSL target
                                     // for MSP430 and MSP432P
#define BSL_CMD_GET_STATUS     0x23  // BSL command header byte that does not 
                                     // require ACK from the BSL target
                                     // for MSP432E

#define BSL_CUSTOM_CMD_LENGTH  1     // custom command length
#define BSL_ACK_LENGTH         1
#define BSL_HEADER_LENGHT      1     // header byte of MSP430 and MSP432P BSL 
                                     // has 1 byte length
#define BSL_LENGTH_LENGTH      2     // length bytes of MSP430 and MSP432P BSL
                                     // has 2 bytes length
#define BSL_CMD_LENGTH         1     // BSL command byte for MSP430 and MSP432P
                                     // has 1 byte length
#define BSL_CRC_LENGTH         2     // CRC bytes of MSP430 and MSP432P 
                                     // has 2 bytes length

#define BSL_E4xx_RESPONSE_GET_STATUS 5
#define BSL_E4xx_RESPONSE_ACK        2

#define RET_ERROR              (-1)
#define RET_OK                 1
#define BSL_UNKNOWN_ERROR_CODE 0x55

/*** UART ********************************************************************/

#define BAUDRATE_1200                     1200
#define BAUDRATE_2400                     2400
#define BAUDRATE_4800                     4800
#define BAUDRATE_9600                     9600
#define BAUDRATE_19200                    19200
#define BAUDRATE_38400                    38400
#define BAUDRATE_57600                    57600
#define BAUDRATE_115200                   115200
#define BAUDRATE_230400                   230400
#define BAUDRATE_460800                   460800
#define BAUDRATE_921600                   921600
#define BAUDRATE_INVOKE_MSP430            9601
#define BAUDRATE_INVOKE_LEGACY_MSP430     9602
#define BAUDRATE_INVOKE_ROCKET            4801
#define BAUDRATE_RESET_SEQUENCE           4802
#define BAUDRATE_INVOKE_9600_MSP432E4XX   9608
#define BAUDRATE_INVOKE_19200_MSP432E4XX  19208
#define BAUDRATE_INVOKE_38400_MSP432E4XX  38408
#define BAUDRATE_INVOKE_57600_MSP432E4XX  57608
#define BAUDRATE_INVOKE_115200_MSP432E4XX 115208
#define BAUDRATE_EXIT_MSP432E4XX          9610

#define BSL_ADDRESS                       0x1000

#define UART_TIMEOUT_CYCLES               100000

// Baudrate indexes
enum baud
{ BAUD_ERROR_PARAMS, 
  BAUD_1200_PARAMS, 
  BAUD_2400_PARAMS, 
  BAUD_4800_PARAMS, 
  BAUD_9600_PARAMS, 
  BAUD_19200_PARAMS,
  BAUD_38400_PARAMS, 
  BAUD_57600_PARAMS, 
  BAUD_115200_PARAMS, 
  BAUD_230400_PARAMS, 
  BAUD_460800_PARAMS, 
  BAUD_921600_PARAMS
};

uint8_t InitUart(uint32_t lBaudrate);
void uartControl();

/*** I2C *********************************************************************/

#define BSL_SLAVE_ADDR                    0x48
#define BSL_SLAVE_ADDR_MSP432E4xx         0x42
#define MAX_SLAVE_ADDR_7BIT               0x007F
#define MAX_SLAVE_ADDR_10BIT              0x03FF

#define BAUDRATE_100KHZ                   100000
#define BAUDRATE_400KHZ                   400000
#define BAUDRATE_INVOKE_100KHZ_MSP430     100001
#define BAUDRATE_INVOKE_400KHZ_MSP430     400001
#define BAUDRATE_INVOKE_100KHZ_MSP432E4XX 100008
#define BAUDRATE_INVOKE_400KHZ_MSP432E4XX 400008
#define BAUDRATE_EXIT_100KHZ_MSP432E4XX   100010
#define BAUDRATE_EXIT_400KHZ_MSP432E4XX   400010

#define I2C_PORT_SEL                      P4SEL
#define I2C_PORT_OUT                      P4OUT
#define I2C_PORT_REN                      P4REN
#define I2C_PORT_DIR                      P4DIR
#define SDA_PIN                           BIT5
#define SCL_PIN                           BIT0
#define SCL_CLOCK_DIV(X)                  (USB_MCLK_FREQ/X)

#define TIMEOUT_RECEPTION  1000000  // Timeout for I2C communication
#define I2C_DELAY_CYCLES   10000    // Delay been used between transmitting and receiving data

// States definition for I2C protocol 
#define SWAIT	    0x00
#define SSTART	    0x01
#define SLENGTH	    0x02
#define SDATA	    0x03
#define SSEND	    0x04
#define SCMD        0x05
#define SDATA_E4XX  0x06

int8_t InitI2C(uint32_t bitrate);
int16_t i2cSendMessage(unsigned char* I2cMessage, int messageLength);
int16_t i2cReceiveMessage(unsigned char* I2cMessage);
unsigned int i2cFsm(unsigned char *dataBuffer);
void i2cControl(void);

/*** SPI *********************************************************************/

#define SPI_CRC_LENGTH                2  // length in byte
#define SPI_DELAY_CLK_PULSE_P4XX      10000
#define SPI_DELAY_CLK_PULSE_E4XX      100

#define BAUDRATE_125KHZ               125000
#define BAUDRATE_250KHZ               250000
#define BAUDRATE_500KHZ               500000
#define BAUDRATE_1MHZ                 1000000
#define BAUDRATE_INVOKE_125KHZ_MSP430 125001
#define BAUDRATE_INVOKE_250KHZ_MSP430 250001
#define BAUDRATE_INVOKE_500KHZ_MSP430 500001
#define BAUDRATE_INVOKE_1MHZ_MSP430   1000001
#define BAUDRATE_INVOKE_125KHZ_MSP432E4XX 125008
#define BAUDRATE_INVOKE_250KHZ_MSP432E4XX 250008
#define BAUDRATE_INVOKE_500KHZ_MSP432E4XX 500008
#define BAUDRATE_INVOKE_1MHZ_MSP432E4XX   1000008
#define BAUDRATE_EXIT_125KHZ_MSP432E4XX 125010
#define BAUDRATE_EXIT_250KHZ_MSP432E4XX 250010
#define BAUDRATE_EXIT_500KHZ_MSP432E4XX 500010
#define BAUDRATE_EXIT_1MHZ_MSP432E4XX   1000010

enum spiStates 
{ idle, 
  TXheader, 
  TXlengthLow, 
  TXlengthHigh, 
  TXreceive, 
  RXheader, 
  RXlengthLow, 
  RXlengthHigh, 
  RXreceive
};

uint8_t spiInit(uint32_t lBaudrate);
void spiControl();

#ifdef __cplusplus
}
#endif

#endif